@@ -17,12 +17,12 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-#  $Id: dpkg-cross.pl,v 1.13 2004/10/19 20:29:51 yoush-guest Exp $
+#  $Id: dpkg-cross.pl,v 1.25 2005/02/23 14:46:58 yoush-guest Exp $
 
 # Determine if the system wide or user defined cross-compile configuration
 # have to be read.
-$conffile = "/etc/dpkg/cross-compile";
-$private_conffile = "$ENV{'HOME'}/.dpkg/cross-compile";
+$conffile = "/etc/dpkg-cross/cross-compile";
+$private_conffile = "$ENV{'HOME'}/.dpkg-cross/cross-compile";
 $conffile = (-e $private_conffile) ? $private_conffile : $conffile;
 
 # Name of the calling application.
@@ -62,6 +62,7 @@ $DPKGCROSSVERSION = "1.18";
 	's390' => 's390-linux',
 	's390x' => 's390x-linux',
 	'ia64' => 'ia64-linux',
+ 	'm32r' => 'm32r-linux',
 	'openbsd-i386' => 'i386-openbsd',
 	'freebsd-i386' => 'i386-freebsd',
 	'darwin-powerpc' => 'powerpc-darwin',
@@ -89,6 +90,7 @@ $DPKGCROSSVERSION = "1.18";
 	's390' => 'ELF 32-bit MSB .* S.390',
 	's390x' => 'ELF 64-bit MSB .* S.390',
 	'ia64' => 'ELF 64-bit LSB .* IA-64',
+ 	'm32r' => 'ELF 32-bit MSB .* M32R',
 	'AR' => 'current ar archive');
 
 # Lists of possible crossprefix'es for each architecture
@@ -106,25 +108,21 @@ $DPKGCROSSVERSION = "1.18";
 	"hppa" => [ "hppa-linux-" ],
 	"s390" => [ "s390-linux-", "s390x-linux-" ],
 	"s390x" => [ "s390x-linux-", "s390-linux-" ],
-	"ia64" => [ "ia64-linux-" ]);
+	"ia64" => [ "ia64-linux-" ],
+	"m32r" => [ "m32r-linux-" ]);
 
-# All environment variables for GCC with their correspondig application name
-# which will be modified for cross-compiling.
-%std_compiler = (
+# All environment variables for compiler and binutils with their corresponding
+# application name which will be modified for cross-compiling.
+# For binutils, only LD and AS make sense: for strip we have
+# a wrapper, ar and ranlib are archiecture-indepndent.
+%std_tools = (
 	CC => "gcc",
 	GCC => "gcc",
 	CXX => "g++",
 	CPP => "cpp",
-	IMAKECPP => "cpp");
-
-# All environment variables for binutils with their correspondig application
-# name which will be modified for cross-compiling.
-%std_binutils = (
-	LD => "ld",
+	IMAKECPP => "cpp",
 	AS => "as",
-	AR => "ar",
-	RANLIB => "ranlib",
-	STRIP => "strip");
+	LD => "ld");
 
 # Contains variable definitions from ``$conffile'' distinguished by their
 # scope ("makeflags" or "environment") of definition.
@@ -201,6 +199,18 @@ sub read_config {
 				$pkgvars{$scope_}{$var_} = $val_;
 			}
 		}
+		elsif (/^unset\s+([\w\d_-]+)$/) {
+			$var_ = $1;
+			
+			$package_ ||= "all";
+			$mode_ ||= "all";
+			$scope_ ||= "makeflags";
+
+			if (($package_ eq "all" || $package_ eq $current_package_) && \
+				($mode_ eq "all" || $mode_ eq $current_mode_)) {
+				$pkgvars{$scope_}{$var_} = {}; # Use a reference as "unset" value
+			}
+		}
 		elsif (/^(package)?\s*?(\S+)\s*:$/i) {
 			$package_ = $2;
 		}
@@ -222,7 +232,7 @@ sub read_config {
 #     Returns the current architecture.
 # return: Current architecture or empty if not set.
 sub get_architecture {
-       return $ENV{'ARCH'} || $arch || $default_arch;
+       return $ENV{'DPKGCROSSARCH'} || $ENV{'ARCH'} || $arch || $default_arch;
 }
 
 
@@ -307,6 +317,8 @@ sub subst {
 	my @defined_vars_ = @_;
 	my ($name_, $newval_);
 
+	return if ref $$_valref_;	# skip "unset" values
+	
 	while ($$valref_ =~ /\$\((\w+)\)/) {
 		$name_ = $1;
 		if (grep "\U$_\E" eq $name_, @defined_vars_) {
@@ -335,7 +347,7 @@ sub subst {
 sub detect_arch {
 	my $file_ = shift (@_);
 
-	my $string_ = `LANG=C file "$file_" 2>/dev/null`;
+	my $string_ = `LC_ALL=C file -L "$file_" 2>/dev/null`;
 	for my $arch_ (keys(%archdetecttable)) {
 		my $re_ = $archdetecttable{$arch_};
 		if ($string_ =~ /$re_/) {
@@ -362,7 +374,7 @@ sub detect_arch {
 
 ################################################################################
 # read_crosstools_file():
-#     Reads /etc/dpkg/crosstools or ~/.dpkg/crosstools and saves results
+#     Reads /etc/dpkg-cross/crosstools or ~/.dpkg-cross/crosstools and saves results
 #     in a global hash. Do nothing if file can't be read.
 # $1: file to read
 my %crosstools; # $crosstools{$arch}{$tool}{$mode}
@@ -403,8 +415,8 @@ sub get_tool {
 	my ($arch_, $tool_, $mode_) = @_;
 
 	if (! $crosstools_read) {
-		read_crosstools_file("/etc/dpkg/crosstools");
-		read_crosstools_file($ENV{"HOME"} . "/.dpkg/crosstools");
+		read_crosstools_file("/etc/dpkg-cross/crosstools");
+		read_crosstools_file($ENV{"HOME"} . "/.dpkg-cross/crosstools");
 		$crosstools_read = 1;
 	}
 
@@ -442,8 +454,9 @@ sub setup_cross_env {
 	read_config ($package, $mode);
 	setup();
 	
-	# Put ``$arch'' into environment through MAKEFLAGS.
+	# Put ``$arch'' into environment
 	$makeflags_{'ARCH'} = $arch;
+	$ENV{'DPKGCROSSARCH'} = $arch;
 	
 	# Also set new `dpkg-architecture' environment veriables.
 	chomp ($tmp_ = `dpkg-architecture -a$arch -qDEB_HOST_ARCH 2>/dev/null`);
@@ -470,13 +483,13 @@ sub setup_cross_env {
 	# Set USRLIBDIR to $(CROSSLIB), for imake-generated Makefiles..
 	$makeflags_{'USRLIBDIR'} = $crosslib;
 	
-	# Set CONFIG_SITE to /etc/dpkg/cross-config.``$arch'', for
+	# Set CONFIG_SITE to /etc/dpkg-cross/cross-config.``$arch'', for
 	# packages using GNU autoconf configure scripts.
-	$makeflags_{'CONFIG_SITE'} = "/etc/dpkg/cross-config.$arch";
+	$makeflags_{'CONFIG_SITE'} = "/etc/dpkg-cross/cross-config.$arch";
 	
-	# Set standard variables for compilers.
-	foreach $var_ ( keys %std_compiler ) {
-		$makeflags_{$var_} = $crossprefix . $std_compiler{$var_};
+	# Set standard variables for compilers and binutils.
+	foreach $var_ ( keys %std_tools ) {
+		$makeflags_{$var_} = $crossprefix . $std_tools{$var_};
 	}
 
 	# Allow to use $crossprefix-gcc -E as preprocessor if $crossprefix-cpp
@@ -485,18 +498,14 @@ sub setup_cross_env {
 		$makeflags_{"CPP"} = $makeflags_{"IMAKECPP"} = $makeflags_{"GCC"} . "\\ -E";
 	}
 	
-	# Set standard variables for binutils (if different CPU).
-	chomp ($native_cpu_ = `dpkg --print-installation-architecture`);
-	if ($arch ne $native_cpu_) {
-		foreach $var_ ( keys %std_binutils ) {
-			$makeflags_{$var_} = $crossprefix . $std_binutils{$var_};
-		}
-	}
-	
 	# Set additional variables specified in "scope makeflags:" for the current
 	# package.
 	foreach $var_ ( keys %{ $pkgvars{'makeflags'} } ) {
-		$makeflags_{$var_} = $pkgvars{'makeflags'}{$var_};
+		if (ref $pkgvars{'makeflags'}{$var_}) {
+			delete $makeflags_{$var_};
+		} else {
+			$makeflags_{$var_} = $pkgvars{'makeflags'}{$var_};
+		}
 	}
 	
 	if (exists $ENV{'MAKEFLAGS'}) {
@@ -516,7 +525,11 @@ sub setup_cross_env {
 	
 	# Set additional environment variabled specified in "mode environment:".
 	foreach $var_ (keys %{ $pkgvars{'environment'} }) {
-		$ENV{$var_} = $pkgvars{'environment'}{$var_};
+		if (ref $pkgvars{'environmrnt'}{$var_}) {
+			delete $ENV{$var_};
+		} else {
+			$ENV{$var_} = $pkgvars{'environment'}{$var_};
+		}
 	}
 }
 
@@ -527,9 +540,16 @@ sub setup_cross_env {
 #     This function also initialise ``$arch'' as set by `dpkg-buildpackage -a'.
 # return: none
 sub dpkgcross_application {
-	exec "$0.orig", @ARGV
-		unless (($arch = $ENV{'ARCH'}) && $ENV{'MAKEFLAGS'} =~ /\bCC=/) ||
-		$arch =~ /^hurd-/;
+	if ($ENV{'DPKGCROSSARCH'}) {
+		$arch = $ENV{'DPKGCROSSARCH'};
+		return;
+	}
+	# Keep this for now for backward-compatibility
+	if ($ENV{'ARCH'} && $ENV{'MAKEFLAGS'} =~ /\bCC=/) {
+		$arch = $ENV{'ARCH'};
+		return;
+	}
+	exec "$0.orig", @ARGV;
 }
 
 ################################################################################
@@ -589,4 +609,34 @@ sub convert_path ($) {
 	return $path
 }
 
+################################################################################
+# convert_ld_library_path():
+#     Remove from LD_LIBRARY_PATH everything that does not start with /usr/lib
+#     or /usr/share. This is needed to avoid non-native libraries in
+#     LD_LIBRARY_PATH. Just unsetting of LD_LIBRARY_PATH does not work because
+#     of usage of LD_LIBRARY_PATH by fakeroot.
+sub convert_ld_library_path () {
+	my @list = split(':', $ENV{'LD_LIBRARY_PATH'});
+	$ENV{'LD_LIBRARY_PATH'} = join(':', grep {$_ =~ /^\/usr\/(lib|share)\//} @list);
+}
+
+################################################################################
+# create_tmpdir():
+#     Safely create a temporary directory
+# $1: Directory basename (random suffix will be added)
+# return: Full directory pathname, undef if failed
+sub create_tmpdir ($) {
+	my ($pd, $d, $b, $try);
+	$b = $_[0];
+	$pd = $ENV{'TMPDIR'};
+	$pd = '/tmp' if ((!$pd) || (! -d $pd));
+	$try = 0;
+	while ($try < 100) {
+		$d = $pd . '/' . $b . '.' . sprintf("%08x", int(rand(0xFFFFFFFF)));
+		return $d if mkdir($d, 0700);
+		$try++;
+	}
+	return undef;
+}
+
 1;
